<?php

/**
 * The admin-specific functionality of the plugin.
 *
 */

/**
 * The admin-specific functionality of the plugin.
 *
 *
 * @package    Settings_Page
 * @subpackage Settings_Page/admin
 */
class Settings_Page_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		add_action('admin_menu', array( $this, 'addPluginAdminMenu' ), 9);   
		add_action('admin_init', array( $this, 'registerAndBuildFields' ));
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */ 
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Settings_Page_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Settings_Page_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		//wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/efas-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Settings_Page_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Settings_Page_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		//wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/efas-admin.js', array( 'jquery' ), $this->version, false );

	}
	public function addPluginAdminMenu() {
		//add_menu_page( $page_title, $menu_title, $capability, $menu_slug, $function, $icon_url, $position );
		add_menu_page(  $this->plugin_name, 'Anti spam', 'administrator', $this->plugin_name, array( $this, 'displayPluginAdminDashboard' ), 'dashicons-welcome-comments', 85 );
		
		//add_submenu_page( '$parent_slug, $page_title, $menu_title, $capability, $menu_slug, $function );
		add_submenu_page( $this->plugin_name, 'Spam log', 'Spam log', 'administrator', $this->plugin_name.'-log.php', array( $this, 'displayPluginAdminSettings' ));
	}
	public function displayPluginAdminDashboard() {
		require_once 'partials/'.$this->plugin_name.'-admin-display.php';
		//require_once 'partials/'.$this->plugin_name.'-log.php';
  }
	public function displayPluginAdminSettings() {
		// set this var to be used in the settings-display view
		$active_tab = isset( $_GET[ 'tab' ] ) ? $_GET[ 'tab' ] : 'general';
		if(isset($_GET['error_message'])){
				add_action('admin_notices', array($this,'settingsPageSettingsMessages'));
				do_action( 'admin_notices', $_GET['error_message'] );
		}
		require_once 'partials/'.$this->plugin_name.'-log.php';
	}
	public function settingsPageSettingsMessages($error_message){
		switch ($error_message) {
				case '1':
						$message = __( 'There was an error adding this setting. Please try again.  If this persists, shoot us an email.', 'my-text-domain' );                 $err_code = esc_attr( 'settings_page_example_setting' );                 $setting_field = 'settings_page_example_setting';                 
						break;
		}
		$type = 'error';
		add_settings_error(
					$setting_field,
					$err_code,
					$message,
					$type
			);
	}
  
	public function registerAndBuildFields() {
			/**
		 * First, we add_settings_section. This is necessary since all future settings must belong to one.
		 * Second, add_settings_field
		 * Third, register_setting
		 */     
		add_settings_section(
			// ID used to identify this section and with which to register options
			'settings_page_general_section', 
			// Title to be displayed on the administration page
			'',  
			// Callback used to render the description of the section
				array( $this, 'settings_page_display_general_account' ),    
			// Page on which to add this section of options
			'settings_page_general_settings'                   
		);
            add_settings_section(
			// ID used to identify this section and with which to register options
			'settings_page_bonus_section', 
			// Title to be displayed on the administration page
			'',  
			// Callback used to render the description of the section
				array( $this, 'settings_page_display_bonus_text' ),    
			// Page on which to add this section of options
			'settings_page_general_settings'                   
		);

      		

		unset($args);
// First field
		$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'text_blacklist',
          'placeholder'      => 'Eric Jones',          
          'name'      => 'text_blacklist',
          'description'      => 'You can add multiple words, one word per line. EX:<br>eric Jones<br>seo expert',
          'required' => 'true',
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'text_blacklist',
			"Don't allow leads that <u>Equal</u> those names",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);
      	unset($args);
// secend field
		$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'emails_blacklist',
          'name'      => 'emails_blacklist',
          'description'      => 'You can add multiple words, one word per line. EX:<br>test@gmail.com<br>ericjonesonline@outlook.com',
          'placeholder'      => 'ericjonesonline@outlook.com',          
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'emails_blacklist',
			"Don't allow leads that Email field <u>Equal</u> those emails",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);
unset($args);
// Text area field
		$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'textarea_blacklist',
          'name'      => 'textarea_blacklist',
          'description'      => '',
          'atter' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'textarea_blacklist',
			"Don't allow leads that textarea field <u>Contain</u> those lines",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);
unset($args);
// Contain links 
		$args = array (
          'type'      => 'input',
          'subtype'   => 'number',
          'id'    => 'contain_links',
          'name'      => 'contain_links',
          'description'      => 'Block sending if contain more then X links',
          'atter' => false,
          'min' => '0',
          'step' => '1', 
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'contain_links',
			"Don't allow leads  if it contains more than X links",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);
unset($args);  
// Ip field
		$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'ip_blacklist',
          'name'      => 'ip_blacklist',
          'description'      => '',
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'ip_blacklist',
			"Don't allow leads that come from this IP address",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);
unset($args);
// country field
	$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'country_blacklist',
          'name'      => 'country_blacklist',
          'description'      => 'See names as here: https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes',
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'country_blacklist',
			"Don't allow leads that come from those Countries",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);  
unset($args);
// tel format
	$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'tel_formats',
          'name'      => 'tel_formats',
          'description'      => 'One format each line,<br> Ex: for having XXX-XXX-XXXX format you will need to add:<br> /[0-9]{3}-[0-9]{3}-[0-9]{4}/<br>',
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'tel_formats',
			"Allow phone numbers in this format only",
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);  
//hebrewneeded     
unset($args); 
            if( get_locale() == 'he_IL'){

		$args = array (
          'type'      => 'input',
          'subtype'   => 'checkbox',
          'id'    => 'hebrewneeded',
          'name'      => 'hebrewneeded',
          'description'      => '',
          'label'      => 0,          
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
		);
          add_settings_field(
              'hebrewneeded',
              'Block forms without any Hebrew character<br><small>(Elemntor forms only)</small>',
              array( $this, 'settings_page_render_settings_field' ),
              'settings_page_general_settings',
              'settings_page_general_section',
              $args
          );  
      }
//error_message
 unset($args); 
		$args = array (
          'type'      => 'input',
          'subtype'   => 'text',
          'id'    => 'error_message',
          'name'      => 'error_message',
          'description'      => '',
          'label'      => 0,          
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
		);
		add_settings_field(
			'error_message',
			'Validtion error message',
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);   
     
unset($args); 
//Sprade love      
		$args = array (
          'type'      => 'input',
          'subtype'   => 'checkbox',
          'id'    => 'sprade_love',
          'name'      => 'sprade_love',
          'description'      => 'Say somebody thet his loved',
          'label'      => 0,          
          'attr' => 'onclick="show_hide_bonus_sec()"',
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'sprade_love',
			'Do you want halp sprade love in the world?',
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		); 
unset($args); 
//spamcounter      
		$args = array (
          'type'      => 'input',
          'subtype'   => 'number',
          'id'    => 'spamcounter',
          'name'      => 'spamcounter',
          'description'      => '',
          'attr'      => "style='display:none;'",          
          'default' => '0',
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'spamcounter',
			'',
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
		);       
unset($args);
      
//error log      
		$args = array (
          'type'      => 'textarea',
          'subtype'   => 'textarea',
          'id'    => 'errorlog',
          'name'      => 'errorlog',
          'description'      => '',
          'attr'      => "style='display:none;'",          
          'default' => '0',
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
					);
		add_settings_field(
			'errorlog',
			'',
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_general_section',
			$args
	);       
unset($args); 
//Add email_to      
		$args = array (
          'type'      => 'input',
          'subtype'   => 'text',
          'id'    => 'add_email_to',
          'name'      => 'add_email_to',
          'description'      => 'Can add moliple emails by sparte in comma(,)',
          'label'      => 0,          
          'attr' => false,
          'get_options_list' => '',
          'value_type'=>'normal',
          'wp_data' => 'option'
		);
		add_settings_field(
			'add_email_to',
			'Send *all* the Elementor forms in your site Also to this Email/s. ',
			array( $this, 'settings_page_render_settings_field' ),
			'settings_page_general_settings',
			'settings_page_bonus_section',
			$args
		);   


// register fields
		register_setting('settings_page_general_settings_page','text_blacklist');
      	register_setting('settings_page_general_settings_page','emails_blacklist');
      	register_setting('settings_page_general_settings_page','textarea_blacklist');
        register_setting('settings_page_general_settings_page','contain_links');
        register_setting('settings_page_general_settings_page','ip_blacklist');
      	register_setting('settings_page_general_settings_page','country_blacklist');
        register_setting('settings_page_general_settings_page','error_message');
      	register_setting('settings_page_general_settings_page','sprade_love');
        register_setting('settings_page_general_settings_page','tel_formats');
        register_setting('settings_page_general_settings_page','add_email_to');

      	// if elementor pro active
      if(in_array('elementor-pro/elementor-pro.php', apply_filters('active_plugins', get_option('active_plugins')))){ 
        if( get_locale() == 'he_IL'){
           register_setting('settings_page_general_settings_page','hebrewneeded');
        }
      }

	}

	public function settings_page_display_general_account() {
		//echo '<p>These settings apply to all Plugin Name functionality.</p>';
	} 
  	public function settings_page_display_bonus_text() {
      	$show = get_option( 'sprade_love' ) ?  "" : 'style="display: none;"';
        echo "<div $show id='bonus-sec'>";
		echo '<hr><h2>Bonus section, Because i love you</h2>';
      	echo '<p>This section not about Spam, Just Setting that i find as usful</p>';

	} 
	public function settings_page_render_settings_field($args) {   
		if($args['wp_data'] == 'option'){
			$wp_data_value = get_option($args['name']);
		} elseif($args['wp_data'] == 'post_meta'){
			$wp_data_value = get_post_meta($args['post_id'], $args['name'], true );
		}

		$description =(isset($args['description'])) ? $args['description'] : '';
        $placeholder = (isset($args['placeholder'])) ? 'placeholder="'.$args['placeholder'].'"' : '';
		$value = ($args['value_type'] == 'serialized') ? serialize($wp_data_value) : $wp_data_value;
		$attr = isset($args['attr']) ? $args['attr'] : "";
        $default =(isset($args['default'])) ? $args['default'] : '';
        $value = $value ? $value : $default;

		switch ($args['type']) {

			case 'input':
					if($args['subtype'] != 'checkbox' && $args['subtype'] != 'select'){
							$prependStart = (isset($args['prepend_value'])) ? '<div class="input-prepend"> <span class="add-on">'.$args['prepend_value'].'</span>' : '';
							$prependEnd = (isset($args['prepend_value'])) ? '</div>' : '';
							$step = (isset($args['step'])) ? 'step="'.$args['step'].'"' : '';
							$min = (isset($args['min'])) ? 'min="'.$args['min'].'"' : '';
							$max = (isset($args['max'])) ? 'max="'.$args['max'].'"' : '';
							if(isset($args['disabled'])){
									// hide the actual input bc if it was just a disabled input the info saved in the database would be wrong - bc it would pass empty values and wipe the actual information
									echo $prependStart.'<input type="'.$args['subtype'].'" id="'.$args['id'].'_disabled" '.$step.' '.$max.' '.$attr.' '.$placeholder.' '.$min.' name="'.$args['name'].'_disabled" size="40" disabled value="' . esc_attr($value) . '" /><input type="hidden" id="'.$args['id'].'" '.$step.' '.$max.' '.$min.' name="'.$args['name'].'" size="40" value="' . esc_attr($value) . '" />'.$prependEnd;
							} else {
									echo $prependStart.'<input type="'.$args['subtype'].'" id="'.$args['id'].'" '.$attr.' '.$step.' '.$max.' '.$placeholder.' '.$min.' name="'.$args['name'].'" size="40" value="' . esc_attr($value) . '" />'.$prependEnd;
							}
							/*<input required="required" '.$disabled.' type="number" step="any" id="'.$this->plugin_name.'_cost2" name="'.$this->plugin_name.'_cost2" value="' . esc_attr( $cost ) . '" size="25" /><input type="hidden" id="'.$this->plugin_name.'_cost" step="any" name="'.$this->plugin_name.'_cost" value="' . esc_attr( $cost ) . '" />*/

					} else if ($args['subtype'] == 'select'){
                      	$label = $args['label'];
                      	$id = $args['id'];
                        $curr_val = $value;;
                        $num = 1;
                      	$the_array = $args['array'];                      
                        if ($label != '') echo '  <label for="'.$id.'">'.$label.'</label>';
                            echo'  <select name="'.$id.'" -id="'.$id.'"  onchange="val()" > ';
                            foreach ($the_array as $key => $value) {
                                echo ' <option value="'.$key.'"';
                                if ($key == $curr_val) {
                                   echo ' selected="selected"';
                                  }else if($num == 1){
                                   echo ' selected="selected"';
                                }
                              	echo '>'.$value.'</option>';
                              $num ++;
                            }
                            echo '</select>';
                    } else {
							$checked = ($value) ? 'checked' : '';
							echo '<input type="'.$args['subtype'].'" id="'.$args['id'].'" '.$attr.' name="'.$args['name'].'" size="40" value="1" '.$checked.' />';
					}
            
					break;
            	case 'textarea':
                    		$placeholder = (isset($args['placeholder'])) ? $args['placeholder'] : '';
            				$has_value = ($value) ? $value : $placeholder;
            				echo '<textarea '.$attr.' id="'.$args['id'].'" name="'.$args['name'].'" rows="5" cols="80">' . esc_attr($has_value) . ' </textarea>';

			default:
					# code...
					break;
		}
      	echo "<small style='display: block;'>$description</small>";
	}
}