<?php
/**
 * Main Elementor validation functions
 *
 */

add_action( 'elementor_pro/forms/validation', 'efas_validation_process' , 10, 2 );
function efas_validation_process ( $record, $ajax_handler ) {
  $spamcounter = get_option( 'spamcounter' ) ? get_option( 'spamcounter' ) : 0;
  $hebrewneeded =  get_option( 'hebrewneeded' );
  $error_message = get_option( 'error_message' ) ? get_option( 'error_message' ) : "Look like a spam, Try again";
    
  // lang check
  if ($hebrewneeded){
    $havehebrew = false;
    $field_id = '1';
    $types = array('text','textarea');
    $all_field_value = "";
    foreach($types as $type){
      $fields = $record->get_field( ['type' => $type] );
      $field = current( $fields );
      $field_value = $field['value'];  
      $field_id = $field['id'];
	  $all_field_value .= " ".$type." = ".$field_value;
      if (efas_is_hebrew($field_value)) {
        $havehebrew = true;
      }
    }
    if(!$havehebrew){
      update_option( 'spamcounter', ++$spamcounter );
      efas_add_to_log($type = "No have hebrew",$all_field_value );
      $ajax_handler->add_error( $field_id, "יש להכניס תוים בעברית" );
    }
  }

  // ip
  $ip = efas_getRealIpAddr();
  $ip_blacklist =  get_option( 'ip_blacklist' );
  // country
  $xml = simplexml_load_file("http://www.geoplugin.net/xml.gp?ip=".efas_getRealIpAddr());
  $countryName = $xml ? strtolower($xml->geoplugin_countryName) : false ;
  $country_blacklist =  strtolower(get_option( 'country_blacklist' ));
  if (strpos($country_blacklist, $countryName) !== false || strpos($ip_blacklist, $ip) !== false ) {
    $ip_country_blacklist = true;
  }
    
    //If country or ip is in blacklist
  if ( $ip_country_blacklist ) {
    update_option( 'spamcounter', ++$spamcounter );
    efas_add_to_log($type = "Country/IP","$countryName/$ip is blacked" );

    die('ip_country_blacklist');
  }

}

// Validate the Text fields.
add_action( 'elementor_pro/forms/validation/text', function( $field, $record, $ajax_handler ) {
	$spamcounter = get_option( 'spamcounter' ) ? get_option( 'spamcounter' ) : 0;
  	$field_value = strtolower($field['value']); 
    if(!$field_value){
      return;
    }
    $error_message = get_option( 'error_message' ) ? get_option( 'error_message' ) : "Look like a spam, Try again";
    $text_blacklist = get_option( 'text_blacklist' ) ? efas_makeArray(get_option('text_blacklist') ) : array('eric jones');
     if( is_array($text_blacklist) ){
       foreach ($text_blacklist as $bad_string) {
        if( efas_is_field_value_equwl_to_string($bad_string, $field_value) ) {
          update_option( 'spamcounter', ++$spamcounter );
          efas_add_to_log($type = "text",$field_value);
          
          $ajax_handler->add_error( $field['id'], $error_message );
        }
       }
    }
}, 10, 3 );

// Validate the Email fields.
add_action( 'elementor_pro/forms/validation/email', function( $field, $record, $ajax_handler ) {
	$spamcounter = get_option( 'spamcounter' ) ? get_option( 'spamcounter' ) : 0;
  	$field_value = strtolower($field['value']); 
    if(!$field_value){
      return;
    }
  
    $error_message = get_option( 'error_message' ) ? get_option( 'error_message' ) : "Look like a spam, Try again";
    $text_blacklist = efas_makeArray( get_option( 'emails_blacklist' ) );
      foreach ($text_blacklist as $bad_string) {
         if( efas_is_field_value_equwl_to_string($bad_string, $field_value) ) {
              update_option( 'spamcounter', ++$spamcounter );
              efas_add_to_log($type = "email",$field_value);
              $ajax_handler->add_error( $field['id'], $error_message );
          }
      }
}, 10, 3 );

// preg_match the Tel field to the given format.
add_action( 'elementor_pro/forms/validation/tel', function( $field, $record, $ajax_handler ) {
  	$spamcounter = get_option( 'spamcounter' ) ? get_option( 'spamcounter' ) : 0;
  	$field_value = strtolower($field['value']); 
    if($field_value == "" || !$field_value ){
      return;
    }
    $error_message = get_option( 'error_message' ) ? get_option( 'error_message' ) : "Look like a spam, Try again";
    $tel_formats = efas_makeArray(get_option( 'tel_formats' ));
    $valid = true;
    if( is_array($tel_formats) ){
      $valid = false;
      foreach ($tel_formats as $format) {
        // Match this format XXX-XXX-XXXX, 123-456-7890 -- like: [0-9]{3}-[0-9]{3}-[0-9]{4}
        if ( preg_match( $format, $field_value ) ) {
          $valid = true;
          break;
        }
      }
      if(!$valid){
         //efas_add_to_log($type = "tel",$field_value);
         $ajax_handler->add_error( $field['id'], $error_message );
      }
    } 
}, 10, 3 );
           
// Validate the Textarea field.
add_action( 'elementor_pro/forms/validation/textarea', function( $field, $record, $ajax_handler ) {
	$spamcounter = get_option( 'spamcounter' ) ? get_option( 'spamcounter' ) : 0;
  	$field_value = strtolower($field['value']); 
    if(!$field_value){
      return;
    }
    $error_message = get_option( 'error_message' ) ? get_option( 'error_message' ) : "Look like a spam, Try again";
    $textarea_blacklist = efas_makeArray(get_option( 'textarea_blacklist' ));
      foreach ($textarea_blacklist as $bad_string) {
        if(efas_is_field_value_exist_in_string($bad_string, $field_value)) {
          update_option( 'spamcounter', ++$spamcounter );
          efas_add_to_log($type = "textarea","field_value include $bad_string");
          $ajax_handler->add_error( $field['id'], $error_message );
        }
    }
  
  	//contain_links
    $max_links = get_option( 'contain_links' );
    if ( $max_links ) {
      $reg_exUrl = "/(http|https)\:\/\/[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(\/\S*)?/";
	  $num_links = preg_match_all( $reg_exUrl, $field_value );
		if ( $num_links >= $max_links ) {
              update_option( 'spamcounter', ++$spamcounter );
              efas_add_to_log($type = "textarea","contain more then  $max_links links");
			  $ajax_handler->add_error( $field['id'], $error_message );
		}
	}

}, 10, 3 );

/// add email adrress bcc to forms
add_filter( 'elementor_pro/forms/wp_mail_headers','efas_add_mail_bcc_to_forms' );
function efas_add_mail_bcc_to_forms( $headers ){

    $add_email_to = efas_makeArray(get_option('add_email_to'));  

	if($add_email_to){
		$add_email_to = implode (", ", $add_email_to);
		$cc_header = 'Cc: ' . $add_email_to . "\r\n";
		$headers .= $cc_header;
	}
	return $headers;
}